; One Button Home - SLAVE Unit obhslv22.ASM  --  2313 micro-controller project

; UNIT #26, 27, 28 (with detection for 25 as "lived-in" section operation)

; ***** / Acts as single touch plate and receives 1/10,000 sec pulse from
; ***** /   other units to control on/off.

; ***** / Pulses for specific touch-plate units are 1/1,000 second apart.

; ***** / 3.58 MHz Xtl for clock

;  - Button Push routine replaces DetectCnt
;  - Beep feedback added for remote switch position 11/8/04
;  - Three AC outlets controlled.  Three sequential locations starting from
;      initial location (5) programmed at SRAM location $6B.

; April 16, 2006, 2005 - 

.include "2313def.inc"

; R16,17 general purpose temporary use register
; R20 contains downcnt.
; R21 contains TPCounter count
; R26 contains TPCount without multiplication factoring in (for use of 10
;   or more touches as alternate to 10X plus individual digits.
; R4 is random number counter for lived-in purposes
; R5, R6 are daylight on time counters
; R7 is random number seconds counter
; R14 is divide by four counter
; R8, R9 are lived-in daylight time store and countdown

;*****Initialization


rjmp	START1
rjmp	START1
rjmp	START1
rjmp	START1
rjmp	ClockCntDwnInt
rjmp	START1
rjmp	START1

	
START1:			;set up stack pointer and initialize cold start

ldi 	r16,low(RAMEND)		; Load low byte address of end of RAM into register R16
out	SPL,R16			; Initialize stack pointer to end of internal RAM

ser	r16		;Set to all ones for out port direction as output
out	DDRB,r16	;Set PORTB to output

clr	r16
out	PortB,r16	;LEDs are off when port pins are low

ldi	R16,250		;Set bit 0, 2 as inputs ... remaining bits as output
out	DDRD,R16	;execute data direction with bit 0 as input
out	PORTD,R16	;execute to set out ports high, and in port with
			; no input resistor

in	r16,TIMSK	;enable 16-bit match to A register
ori	r16,64		
out	TIMSK,r16

in	r16,TCCR1B
ori	r16,11		; divide by 64 on prescaler
out	TCCR1B,r16	;enable reset of counter to zero (bit3) after match detection.
			;and bits 0,1,2 select prescaling source as 64 of clock

ldi	r16,218
out	OCR1AH,r16
ldi	r16,123		;default selected at 123 for this particular crystal
out	OCR1AL,r16	;loads the compare register A for timer 1 (1 second)
			; -- clock adj. to run slow (for Corr Purp)

clr	r30		;clear delay-on counter
clr	r31		;clear delay-on counter

clr	r18		;zero all flags in register r18

ldi	r16,50
sts	$6B,r16			;Unit number 25  (unit number times 2)
				; 26, 27, 28 to follow as sequential other relay numbers

clr	r16
sts	$AE,r16
sts	$AF,r16			;zero delay-on counter

bclr	7			;disable global interrupts
				; can be turned on later for test purposes
				
rcall	Delay2310		;let touch plate capacitor charge

;------------------------------------



MainLoop:

sbrc	r18,3			;skip if bit 3 (8) is clear (continue in main loop)
rjmp	ComputerLoop

sbis	PIND,0			;skip if pin 0, port D touch plate is high			; is NOT touched (set)
rjmp	DecodeMLTouchPlate	

sbis	PIND,2			;skip if bit 2(4), pwr/data is set (no pulse)(2T) 
rjmp	GiveCommands			;check for timed unit pulse

rjmp	MainLoop

;-------------------------------------------



ComputerLoop:


sbic	PIND,0		;skip if touch plate touched
rjmp	MainLoop

rcall	ButtonPushTwo

rjmp	MainLoop



;----------------------------

DecodeMLTouchPlate:

rcall	ButtonPush	;check for entry

cpi	r20,1		;checks to see if LONG Count has been recorded
brsh	DML1		;branch if same or higher than 1
rjmp	TPShortCount


DML1:

rjmp	TPLongCount

;-----------------------------------------------

			
TPShortCount:		;decode short counts


TPSC1:

cpi	r21,1		;#1 short = Togle AC relay 1 on/off
breq	TPSC1a
rjmp	TPSC2


TPSC1a:

rcall	ToggleAC
rjmp	TPCEnd

;------------------------------


TPSC2:

cpi	r21,2		;toggle AC relay 2
breq	TPSC2a
rjmp	TPSC3


TPSC2a:

rcall	ToggleAC2
rjmp	TPCEnd

;--------------------------------


TPSC3:

cpi	r21,3		;toggle AC relay 3
breq	TPSC3a
rjmp	TPSC10


TPSC3a:

rcall	ToggleAC3	;toggle AC relay with preset lived-in timings
rjmp	TPCEnd

;---------------------------------------------


TPSC10:			;Time on commands for Relay 1

cpi	r26,10		;ONE hour on time for unit 1 relay (using r26)
breq	TPSC10a		
rjmp 	TPSC11


TPSC10a:

ldi	r16,16
sts	$AE,r16
ldi	r16,14
sts	$AF,r16		;ONE hour on time calculation
rjmp	TPSOut


TPSC11:

cpi	r26,11		;THREE hours on time for unit 1 relay (using r26)
breq	TPSC11a		
rjmp 	TPSC12


TPSC11a:

ldi	r16,48
sts	$AE,r16
ldi	r16,42
sts	$AF,r16		;THREE hours on time calculation for relay 1
rjmp	TPSOut


TPSC12:

cpi	r26,12
breq	TPSC12a
rjmp	TPSC13


TPSC12a:

ldi	r16,32
sts	$AE,r16
ldi	r16,253
sts	$AF,r16		;EIGHTEEN hours on time for relay 1
rjmp	TPSOut


TPSC13:
cpi	r26,13
breq	TPSC13a
rjmp	TPSC14


TPSC13a:

sbr	r18,1		;set bit 0(1), lived-in flag to on
cbr	r18,2		;clear bit 1(2), night light flag

sbi	PortB,4		;on Sonalert - beep sonalert once
rcall	Delay165
cbi	PortB,4		;off Sonalert

rjmp	TPCEnd


TPSC14:

cpi	r26,14
breq	TPSC14a
rjmp	TPSC15


TPSC14a:

sbr	r18,2		;set bit 1(2), night light flag
cbr	r18,1		;clear bit 0(1), lived-in flag to off

sbi	PortB,4		;on Sonalert - beep sonalert twice
rcall	Delay165
cbi	PortB,4		;off Sonalert
rcall	Delay495
sbi	PortB,4		;on Sonalert
rcall	Delay165
cbi	PortB,4		;off Sonalert

rjmp	TPCEnd


TPSC15:

cpi	r26,15		;night light only operation of relay 3
breq	TPSC15a
rjmp	TPCOther



TPSC15a:

cbr	r18,3		;clear bits 0(1) lived-in and 1(2) night light flags
rjmp	TPCEnd



TPSout:

sbi	PortB,2		;turn on relay 1

rjmp	TPCEnd


TPCOther:

; All other numbers will toggle remote units

lsl	r21		;multiply by two
mov	r13,r21

sbr	r18,16		;set bit 4(16), immediate flag to on
sts	$69,r13		;store command into $69, immediate command store

rjmp	TPCEnd



TPCEnd:

sbi	PortB,0		;on red led
rcall	Delay165
cbi	PortB,0		;blink red led once

rjmp	MainLoop


;----------------------------


TPLongCount:

cpi	r20,10		;Is long count >=10 ? (Enters command mode)
brsh	TPLC1

cpi	r20,3		;Is long count >=3 ?  (Turn another switch on/off)
brsh	TPLC2

rjmp 	TPCEnd


TPLC1:			;sets flag for command mode

sbr	r18,8		;set bit 3(8) for command mode on


rjmp	TPCEnd


TPLC2:			;Turn another switch off

rcall	OneREDFlash
rcall	ButtonPush
cpi	r20,1		;Exit if count is not short count
brsh	TPCEnd


TPLC2a:

cpi	r21,99		;checks for maximum number
brlo	TPLC2b
rjmp	TPCEnd


TPLC2b:

cpi	r21,6		;check for unit #3 off command
brne	TPLC2b1

TPLC2b1:

lsl	r21		;multiply by two
mov	r13,r21

inc	r13		;it is off command so extra cycle added


TPCL2c:

sbr	r18,16		;set bit 4(16), immediate flag to on
sts	$69,r13		;store command into $69, immediate command store

rjmp	TPCEnd


;-----------------------------


GiveCommands:

sbrs	r18,4		;skip if bit 4(16), immediate command is set
rjmp	GC5


GC1:

lds	r19,$69		;get immediate command from SRAM
cbr	r18,16		;clear immediate flag, bit 4(16)


GC4:			;delay the pulse approx. one-half cycle to positiion

ldi	r16,250		


GC4a:

sbis	PIND,0		;skip if touch plate not touched
rjmp	MainLoop	

dec	r16
brne	GC4a

ldi	r16,182

GC4b:

dec	r16	
brne	GC4b

;--------------------------------


GC4c:			;Issue number of delay cycles

clr	r16
ldi	r17,2		;established delay number

GC4d:

sbis	PIND,0		;skip if touch plate not touched
rjmp	MainLoop

nop
nop

dec	r16
brne	GC4d

dec	r17
brne	GC4d


GC4e:

dec	r19
brne	GC4c

rcall	UsecPulse	;issue 50 usec pulse for unit command



;------------------	;This secion looks for "ON," 2nd pulse
			;IF detected, the Sonalert beeps out unit number

ReceiveResultPulse:
			
clr	r16		;delay of 286 usec before searching for pulse

RRP1:

nop
dec	r16
brne	RRP1		;execute 286 usec delay before continuing
			
clr	r16


RRP2:

sbis	PIND,2		;skip if data line pulse set -- not present
rjmp	BeepUnitNo
nop
nop
dec	r16
brne	RRP2		;500 usecond window to detect second pulse

rjmp	SyncUp


BeepUnitNo:		;Beep unit number out on Sonalert
			;Part of PCB routine

lds	r19,$69		;get immediate command unit number
lsr	r19		;divide by two to get correct unit number


BUN1:

cpi	r19,10
brsh	BUN5


BUN2:

tst	r19
breq	SyncUp		;jump out - beep out of number now complete

dec	r19
sbi	PortB,4		;on Sonalert
rcall	Delay165
cbi	PortB,4		;off Sonalert
rcall	Delay495

rjmp	BUN2


BUN5:		;decrement by 10

subi	r19,10

sbi	PortB,4		;On sonalert
rcall	Delay495
cbi	PortB,4
rcall	Delay495	; generate long beep for 10s count

rjmp	BUN1

;---------------------

SyncUp:			;Assures proper timing resumption for next data pulse
			;Part of PCB routine

sbic	PIND,2		;skip if pulse bit clear (on)
rjmp	SyncUp


SyncUp1:

clr	r16		;set up counter
ldi	r17,199		;.1 second plus delay


SyncUp2:

dec	r16
brne	SyncUp4

dec	r17
brne	SyncUp4		; > .1 second countdown time

rjmp	MainLoop		;jump out of routine - time is now correct


SyncUp4:

sbic	PIND,2		;skip if D,2 is clear (on)
rjmp	SyncUp2		;continue count down

rjmp	SyncUp1		;reset to zero and start count over again


;--------


GC5:			;Relay Delay On Countdown

lds	r30,$AE
lds	r31,$AF		;get delay-on count from SRAM

mov	r16,r30
or	r16,r31
breq	PCD		;bypass decrement of Z register if zero


sbiw	r30,1		;decrement r30,31, Z register
sts	$AE,r30
sts	$AF,r31

or	r30,r31
brne	PCD		;off relay if zero

cbi	PortB,2		;off relay


PCD:			;Power Command Line Decode

push	r28
push	r29

lds	r19,$6B		;get unit number (unit number times 2)
clr	r28		;zero Z register
clr	r29


PCD1:			;START of decode process

clr	r16
ldi	r17,2


PCD2:

sbis	PIND,0		;skip if touch plate not touched
rjmp	PCDEnd

nop	
nop

dec	r16
brne	PCD2

dec	r17	
brne	PCD2

dec	r19		;number of unit number loops
brne	PCD1


PCD3:

adiw	r28,1		;increment on/off relay count (1)

sbis	PIND,0		;skip if bit 0(1) is set - button not pushed
rjmp	PCDEnd

sbis	PIND,2		;skip if bit 2(4), pwr/data line is set (no pulse)(2T) 

rjmp	PCD4		;pulse is detected (2)

cpi	r29,9		;max number for count (1) - unit #25 decode limit
brne	PCD4e		;jump out because of max count (1F)

cpi	r28,92		;low bits max count
brsh	PCDd

rjmp	PCD3		;loop back(2)


PCDd:

rjmp	PCDEnd


PCD4e:			;timing makeup

nop
rjmp	PCD3

;-------------------------------------------------


PCD4:			;Three Relay Decode

clr	r16
sts	$AE,r16
sts	$AF,r16		;clear count in delay on counter

sts	$6C,r28		;store low bytes of count
sts	$6D,r29		;store high bytes of count

adiw	r28,63		;adi to word 213 (256-43) - steps count ahead
adiw	r28,63		
adiw	r28,63
adiw	r28,24

cpi	r29,2		;  high byte will be incremented if too high
brlo	PCD4y
rjmp	PCD4n


PCD4y:			;dusk has set in

sbr	r18,32		;set bit 5(32), night period flag
sbrs	r18,1		;skip if bit 1(2),night light flag, is set
rjmp	PCD4y1

sbi	PortB,6		;on relay 3
rjmp	PCDEnd		;exit


PCD4y1:

sbrs	r18,0		;skip if bit 0(1), lived in flag, is set
rjmp	PCDEnd

sbr	r18,4		;set bit 2(4), lived in LIVE flag

ldi	r16,1		;set up randopm delay of up to 4.2 minutes 
sts	$73,r16		; after dusk for ON of lamp
sts	$72,r7		

ldi	r16,3		;two plus one
sts	$70,r16
sts	$71,r16		;put in approx 8.5 minutes time into on counter

mov	r8,r5
mov	r9,r6		;save night on time in r8,9

clr	r5
clr	r6		;clear values in r5,6 to start new count

mov	r16,r9
cpi	r16,27		;is it 2 hours or less? (less than 27)
brlo	PCD4y2

cpi	r16,84		;is is same or greater than 6 hours?  (21 or more)
brsh	PCD4y2
rjmp	PCDEnd


PCD4y2:

ldi	r16,56		;put about 4 hours into cycle timer (sec/4)
mov	r9,r16
clr	r8
rjmp	PCDEnd


PCD4n:

lds	r28,$6C
lds	r29,$6D

adiw	r28,63		;adi to word 169 (256-87) - step count ahead
adiw	r28,63
adiw	r28,43

cpi	r29, 3
brlo	PCD4za
rjmp	PCD4p


PCD4za:			;clear night period flag and off relay 3 (if necessary)

cbr	r18,32		;clear night period flag, r5(32)
sbrs	r18,1		;skip if bit 1(2), night light flag, is set
rjmp	PCDEnd

cbi	PortB,6		;OFF relay #3 (dawn has come)

rjmp	PCDEnd


PCD4p:

lds	r28,$6C
lds	r29,$6D

adiw	r28,63		;adi to word, 126 (256-130)
adiw	r28,63

cpi	r29,4
brlo	PCD4q
rjmp	PCD4r


PCD4q:

rcall	ToggleAC	;toggle relay 1
sbis	PortB,2		;skip if relay 1, PortB,2 is set (on)
rjmp	PCDEnd
rjmp	SendResultPulse


PCD4r:

lds	r28,$6C
lds	r29,$6D

adiw	r28,63		;adi to word, 82 (256-174)
adiw	r28,19

cpi	r29,5
brlo	PCD4s
rjmp	PCD4t


PCD4s:

cbi	PortB,2		;off relay 1
rjmp	PCDEnd


PCD4t:

lds	r28,$6C
lds	r29,$6D

adiw	r28,39		;adi to word, 39 (256-217) 
cpi	r29,6
brlo	PCD4u
rjmp	PCD4v


PCD4u:

rcall	ToggleAC2	;toggle relay 2
sbis	PortB,5		;skip if relay 2, PortB,5 is set (on)
rjmp	PCDEnd
rjmp	SendResultPulse


PCD4v:

lds	r28,$6C
lds	r29,$6D

adiw	r28,63		;adi to word, 251 (256-5) 
adiw	r28,63
adiw	r28,63
adiw	r28,62
cpi	r29,8
brlo	PCD4w
rjmp	PCD4x


PCD4w:

rcall	LightOnOffCounterLoad	;set lived-in timers to a random value
cbi	PortB,2			;off relay 2
rjmp	PCDEnd


PCD4x:

lds	r28,$6C
lds	r29,$6D

adiw	r28,63		;adi to word 208 (256-48) - step count ahead
adiw	r28,63
adiw	r28,63
adiw	r28,19

cpi	r29,9		;one more than 8 to detect 8 (brlo)
brlo	PCD4m
rjmp	PCD4z


PCD4m:			;Toggle Relay 3

rcall	ToggleAC3
sbis	PortB,6		;skip if relay 3, PortB,6 is set (on)
rjmp	PCDEnd
rjmp	SendResultPulse


PCD4z:

lds	r28,$6C
lds	r29,$6D

adiw	r28,63		;adi to word 164 (256-92) - step count ahead
adiw	r28,63
adiw	r28,38

cpi	r29,10		;one more than 9 (to detect 9)
brlo	PCD4o
rjmp	PCDEnd


PCD4o:			;off relay 3

cbi	PortB,6		;off relay 3
rjmp	PCDEnd


;------			;This section adds second pulse IF relay is ON

SendResultPulse:


clr	r16		;provides 430 usec delay before 2nd pulse is sent


SRP1:

nop	
nop
nop
dec	r16
brne	SRP1		;loop back until 430 usec delay is fulfilled

rcall	UsecPulse	;send out 2nd pulse indicating this circuit is ON


SRP2:

rcall	DelayPoint1	; .1 second delay
rjmp	PCDEnd


;---------------------------


PCDEnd:

inc	r7		;increment random number counter

pop	r29
pop	r28



;--------------
;BELOW IS LIVED IN SECTION
;--------------



sbrs	r18,5		;skip if bit 5(32), night period flag, is set
rjmp	LIS1

dec	r14		;decrement four counter
brne	LIS1

ldi	r16,4
mov	r14,r16		;re-instate four count

inc	r5
brne	LIS1

inc	r6


LIS1:

sbrs	r18,2		;skip if bit 2(4), lived in LIVE flag is set
rjmp	LISEnd		;exit

dec	r8
brne	LIS2

dec	r9
brne	LIS2

cbr	r18,4		;clear bit 2(4), lived in LIVE flag
cbi	PortB,6		;off relay 3

rjmp	LISEnd


LIS2:

sbis	PortB,6		;skip if bit 6(relay3) is set - light is on
rjmp	LIS3		;light is off


LIS4:			;light is on

lds	r16,$70
dec	r16
sts	$70,r16		;get light on time and check for zero
brne	LISEnd	

lds	r16,$71
dec	r16
sts	$71,r16
brne	LISEnd		;decrement light on timer

rcall	LightOnOffCounterLoad

cbi	PortB,6		;off relay 3
rjmp	LISEnd


LIS3:			;light is off

lds	r16,$72
dec	r16
sts	$72,r16
brne	LISEnd	

lds	r16,$73
dec	r16
sts	$73,r16
brne	LISEnd		;decrement light off counter

rcall	LightOnOffCounterLoad

sbi	PortB,6		;on relay 3


LISEnd:

rjmp	MainLoop


;--------------------------------

LightOnOffCounterLoad:	;loads on and off times into lived-in on and off timers

mov	r16,r7		;set up new times for light OFF counter
andi	r16,3		;save two LSD of random counter, r7, for processing
mov	r10,r16		;temporarily save in r10

ldi	r16,6		;minimum of 21 minutes off time (+1)
add	r16,r10		;add in random number from 0 to 3

sts	$73,r16

sts	$72,r7		;put random R7 value into lower bits of off counter


mov	r16,r7		;set light ON counter - get random value
andi	r16,3		;save two LSDs 
mov	r10,r16		;save temporarily in r10

ldi	r16,2		;minimum of 4.5 minutes on (1) (+1)
add	r16,r10		;add in value in r10

sts	$71,r16		

sts	$70,r7		;put random value in r7 as LS bits

ret

;--------------
;END of Lived In Section
;-------------


;----------------------------


ButtonPush:	;Replaces old DetectCnt - Updated 2/9/2005

push	r23
push	r30


BP12:

clr	r23		;set up > 2 minute delay timeout
ldi	r16,8		;(seven full cycles)


BP12a:

sbis	PIND,0		;skip if touch plate not touched
rjmp	BP17		;recycle until button is pushed


rcall	Delay40th	;1/40th second delay routine

dec	r23		;delay of >2 minutes in event touch plate abandoned
brne	BP12a

dec	r16
brne	BP12a

cbr	r18,8		;clear bit 3(8) - flag (return to main loop)
rjmp	BPEnd


BP17:

clr	r20		;hold down count counter
clr	r21		;dit dah count counter
clr	r26		;clear r26, single dit counts past nine count


BP15:

ldi	r23,80	; Two second wait delay = 80 (delay for Down Count)


BP1:

sbic	PIND,0	;skip if port D, bit 2, touch plate input, is low.
rjmp	BP11

rcall	Delay40th

dec	r23
brne	BP1


BP3:

sbi	PortB,0		;flash RED LED
rcall	Delay165
cbi	PortB,0

inc	r20		;step long count one count

ldi	r23,30		;reset to less than 1 sec down count
rjmp	BP1


BP11:

cpi	r23,62		;compare count in r23 for long touch
brlo	BP10		;branch if lower (long touch - dah)

inc	r21		;step short count one count - a dit short touch command	
inc	r26		;single count counter for counts past nine

rjmp	BP9


BP8:

ldi	r23,50		;reset to 1 1/4 second pause between touches count
rjmp	BP5


BP10:			;increment by 10 because it is dah, long touch

ldi	r16,10
add	r21,r16


BP9:

cpi	r21,100		;be sure count in r21 is below 100
brlo	BP8

ldi	r21,0		;zero r21 - count is invalid
rjmp	BP8


BP5:

rcall	Delay40th
dec	r23
brne	BP6


BP16:

tst	r20		;test for zero
breq    BPEnd

clr	r21		;zero r21 & r26 since r20 has holddown count data
clr	r26		; (only one register should contain data)
rjmp	BPEnd


BP6:

sbic	PIND,0	;skip if port D, bit 2, touch plate input, is low.
rjmp	BP5


BP7:

rcall	Delay40th
rjmp	BP15


BPEnd:

pop	r30
pop	r23

ret

;----------------------------


ButtonPushTwo:		;includes string of functions requiring attention

rcall	ButtonPush	
	
			;END of ButtonPush --- now comes the decoding

cpi	r20,1		;checks to see if LONG Count has been recorded
brsh	LongCount	;branch if same or higher than 1

; The following are short counts

ShortCount:


rjmp	LCCEnd


;-----------------------------



LongCount:		;series for long counts



;-----------------------


LCC3:			;Enter new unit number

cpi	r20,3
breq	LCC3a
rjmp	LCC10			;Set new default number for Unit Number


LCC3a:

push	r12

clr	r12
rcall	TwoRedFlash	;Signal that two entries are needed

rcall	ButtonPush
rcall	Mult10		;multiply r20 value by 10
add	r12,r20		;add in tens units

rcall	OneRedFlash	;indicate an entry is needed
rcall	ButtonPush
add	r12,r20		;add in LSD of count -- data is now complete

lsl	r12		;multiply value by TWO
sts	$6B,r12		;store new unit number in $6B (1 to 50)

pop	r12

rjmp	LCCEnd


LCC10:			;Set LSB for compare register 3.58 countdown

cpi	r20,10
breq	LCC10a
rjmp	LCC15

LCC10a:

rcall	SetCompareRegister	;set 3.58 clock compare register during interrupts

rjmp	LCCEnd


LCC15:			;turn interrupts on (1HC)
			; test purposes only - blinks red led twice at minute,
			; once every second
cpi	r20,15		;Any touch releases this routine
breq	LCC15a
rjmp	LCCEnd


LCC15a:

bset	7		;turn on interrupts


LCC15b:

sbic	PIND,0 		;skip if touch plate is touched - a jump out
rjmp	LCC15b

rcall	ButtonPush
bclr	7		;turn off interrupts
rjmp	LCCEnd	


;--------------------------------


LCCEnd:

cbr	r18,8		;clear bit 3(8) for return to main loop

ldi	r16,6
rcall	RapidRedFlash

ret

;---------------------------------------


ClockCntDwnInt: 		;interrupt to update clock counters

push	r16
push	r30
push	r31

in	r16,SREG	;save flags
sts	$60,r16

sbr	r18,4		;set bits 2 (4) -- second flag 
			;(set each second)

CC1b:

sbi	PortB,0		;on RED LED
rcall	Delay165
cbi	PortB,0		;off RED LED

inc	r15			;step one count of SECONDS counter
mov	r16,r15

cpi	r16,60
breq	CCDI1

rjmp	CCDIExit		;return since no action required


CCDI1:

clr	r15		;zero r15 -- SECONDS counter

sbr	r18,64		;set bit 6(64) -- the NEW MINUTE flag


BlinkLED:		;double blink at each new minute		

rcall	Delay165
sbi	PortB,0		;on RED LED
rcall	Delay165
cbi	PortB,0		;off RED LED



CCDI1Cntr:

lds	r16,$61		;load low minutes byte into R16

inc	r16
sts	$61,r16

brne	CCDI1a		;jump out if not zero (no overflow)

lds	r16,$62		;high byte minutes
inc	r16		;increment high byte minutes
sts	$62,r16


CCDI1a:

lds	r16,$62		;check high byte if it is 5
cpi	r16,5

breq	CCDI1b

rjmp	CCDIalarmExit


CCDI1b:			

lds	r16,$61
cpi	r16,160		;1440 minutes reached -- it's now midnight

breq	CCDI1c

rjmp	CCDIalarmExit


CCDI1c:			;zero high and low bytes - it is midnight

clr	r16
sts	$61,r16
sts	$62,r16

lds	r16,$63
inc	r16
sts	$63,r16

cpi	r16,8

breq	CCDI1d


CCDI1e:

rcall	ClockCorrection	;call clock correction (midnight corr)

rjmp	CCDIalarmExit


CCDI1d:

ldi	r16,1		;load with 1 (Sunday)
sts	$63,r16
rjmp	CCDI1e		;to continue


CCDIalarmExit:

;rcall	CheckAllAlarms	



CCDIExit:		;Exit this interrupt routine

lds	r16,$60		;SREG temporary store
out	SREG,r16	;restore SREG data

pop	r31
pop	r30
pop	r16

reti


;-----------------------------


ClockCorrection:	

lds	r16,$66
tst	r16		;if zero, no correction is needed
			;1= every day, 2= every other day, etc.

breq	CCExit		;exit routine with no action

dec	R16
sts	$66,r16		;store new value in variable

breq	CC3		;it is time for update of second counter

ret


CC3:

lds	r16,$64		;seconds correction from permanent store
mov	r15,r16		;put in seconds store area to correct time

lds	r16,$65		;get permanently stored value for interval correction
sts	$66,r16		;store new value in variable


CCExit:

ret

;----------------------------------------------


ToggleAC:		;toggles AC power on/off

sbis	PORTB,2		;skip if bit 2 (AC) is set
rjmp	TAC1

cbi	PortB,2
ret

TAC1:

sbi	PortB,2		;set bit 2 (on power)
clr	r16
sts	$AE,r16
sts	$AF,r16		;zero count for delay-on count

ret

;--------------------------


ToggleAC2:		;toggles AC power to relay 2 on/off


sbis	PortB,5
rjmp	TAC2b

cbi	PortB,5		;off power
ret


TAC2b:

sbi	PortB,5		;set bit 5 (on relay 2 power)
ret


; -------------------------------------


ToggleAC3:		;Toggle relay 3 on/off

rcall	LightOnOffCounterLoad	;set on/off times for lived-in counters

sbis	PortB,6
rjmp	TAC3b

cbi	PortB,6		;clear bit 6 - off relay 3
ret


TAC3b:

sbi	PortB,6		;set bit 6 - on relay 3
ret


; -------------------------------------

SetCompareRegister:

		;puts new value into compare register A.  
		;Default = 218 for high byte, 140 for low byte.
		
push	r1

clr	r1
rcall	TwoRedFlash
rcall	OneRedFlash	;total of three flashes indicating multiple inputs

rcall	ButtonPush
rcall	Mult100		;multiply r20 value by 100
mov	r1,r20		;set 100's column

rcall	OneRedFlash	;indicate an entry is needed
rcall	ButtonPush
rcall	Mult10		;multiply r20 value by 10
add	r1,r20		;add in tens units

rcall	OneRedFlash	;indicate an entry is needed
rcall	ButtonPush
add	r1,r20		;add in LSD of count -- data is now complete

ldi	r16,218
out 	OCR1AH,r16
out	OCR1AL,r1	;put new compare register A data in place

pop	r1
ret

;-------------------------------------

SecondsCorrection:	;Set seconds of correction and how often (2 entries)

rcall	TwoRedFlash
rcall	ButtonPush
sts	$64,r20		;seconds of correction (zero to 255)

rcall	OneRedFlash
rcall	ButtonPush	;enter interval from 0 to 255 - zero = no correction
sts	$65,r20
sts	$66,r20		;put value both in perm and temp store areas

ret

;---------------------


FlashRLED:		;Flash Green LED no. of time in R16 (non-interruptible)

push	r0
mov	r0,r16

tst	r16
breq	FlashRLEDend

FlashRLEDa:

sbi	PORTB,0			;Turn on RED LED
rcall	Delay165
cbi	PORTB,0			;Off RED LED
rcall	Delay495
dec	r0
brne	FlashRLEDa

FlashRLEDend:

pop	r0

ret

;------------------------------------

OneREDFlash:

ldi	r16,1
rcall	FlashRLED
ret

;------------------------------------


TwoREDFlash:

ldi	R16,2		;prepare for two flashes
rcall	FlashRLED
ret

;-------------------------------------


RapidRedFlash:	;r16 contains number of rapid red flashes

RRF1:

sbi	PortB,0		;on red LED
rcall	Delay165
cbi	PortB,0		;off red led
rcall	Delay165

dec	r16
brne	RRF1

ret	


;-------------------------------------


IGFlash:			;Interruptible green flash - R16  contains 
				; number of flashes

tst	r16
breq	IGF1
sbi	PortB,1		;on Green LED
rcall	IDelay330
cbi	PortB,1		;off Green LED
rcall	IDelay495
dec	r16
rjmp	IGFlash

IGF1:
rcall	IDelay1155
ret				

;----------------------------


IRFlash:			;Interruptible red flash - R16  contains 
				; number of flashes

tst	r16
breq	IRF1
sbi	PortB,0		;on Red LED
rcall	IDelay330
cbi	PortB,0		;off Red LED
rcall	IDelay495
dec	r16
rjmp	IRFlash

IRF1:
rcall	IDelay1155
ret				


;---------------------------


;Delay1 subroutines


DelayPoint1:		;.1 second delay

push 	r1
push 	r16

clr	r1
ldi	r16,233


DP1a:

nop
nop
nop
dec	r1
brne	DP1a
dec	r16	
brne	DP1a

pop	r16
pop	r1

ret

;--------------------


Delay2310:

push	r31
ldi	r31,24
rjmp	DelayStart


Delay40th:		; 1/40th second delay
			; r31 is a reserved variable
push	r31
push	r0
push	r1

inc	r7		;increment random number counter by one

ldi	r31,116
mov	r1,r31		; set up r1 for 1/40th second delay
ldi	r31,1
clr	r0

rjmp	DelayA


Delay1155:		;Delay series with r31 containing zero value at start
			; r31 is a reserved variable
push	r31
ldi	r31,12
rjmp	DelayStart


Delay495:

push	r31
ldi	r31,6
rjmp	DelayStart


Delay330:

push	r31
ldi	r31,2
rjmp	DelayStart


Delay165:

push	r31
ldi	r31,1
rjmp	DelayStart


DelayStart:

push	r0
push	r1

clr	r0
clr	r1		;zero registers

DelayA:

dec	R0
brne	DelayA

dec	r1
brne	DelayA

dec	r31
brne	DelayA

pop	r1
pop	r0
pop	r31

ret

;------------------------

IDelay2310:		;interruptible delay routines

push	r31
ldi	r31,24
rjmp	IDelayStart


IDelay1155:

push	r31
ldi	r31,12
rjmp	IDelayStart


IDelay495:

push	r31
ldi	r31,6
rjmp	IDelayStart

IDelay330:

push	r31
ldi	r31,2
rjmp	IDelayStart

IDelay165:

push	r31
ldi	r31,1


IDelayStart:

push	r0
push	r1

clr	r0		;zero
clr	r1		;zero	

IDelayA:

dec	R0
brne	IDelayA

push	r16


IDS5:

sbis	PIND,0		;Skip if button not pushed
rjmp	IDelayB
pop	r16

dec	R1
brne	IDelayA

dec	r31	
brne	IDelayA
rjmp	IDelayC


IDelayB:

cbi	PortB,0		;off red and green LEDs
cbi	PortB,1

rcall	ButtonPushTwo	;new function service is required from button
pop	r16

IDelayC:
	
pop	r1
pop	r0
pop	r31

ret

;--------------------------


Mult100:		;Multiplies value in R20 by 100 and returns value in R20

push	r1
clr	r1	;zero r1 for upcoming data

tst	r20
breq	Mult100b

ldi	r16,100	;the multiplication value


Mult100a:

add	r1,r16
dec	r20
brne	Mult100a


Mult100b:

mov	r20,r1	;for return of value
pop	r1
ret


;--------------------------------------

Mult10:		; multiplies value in R20 by 10 returning value in R20

push 	r1
clr	r1		;zero R1

tst	R20		;test long count value
breq	Mult10b		;value is zero

ldi	r16,10		;put ten into r16

Mult10a:

add	r1,r16		;add ten for each value in R1
dec	r20
brne	Mult10a

Mult10b:

mov	r20,r1		;move r1 value into r16
pop	r1
ret


;-----------------------------

UsecPulse:		; 50 microsecond pulse for power/data line

ldi	r16,60		;load for 50 usec pulse width
sbi	PortB,3		;pulse on

UsecPulse1:

dec	r16
brne	UsecPulse1

cbi	PortB,3		;pulse off

ret


;----------------------------------------------

BeepSonalert:		;Used for TESTING

dec	r16		;beeps sonalert r16 MINUS ONE times
breq	BeepSonalert1

sbi	PortB,4		;on Sonalert
rcall	Delay495
cbi	PortB,4		;off Sonalert
rcall	Delay495
rjmp	BeepSonalert


BeepSonalert1:

sbic	PIND,2		;skip if bit PD2 is clear (sync up to pulse)
rjmp	BeepSonalert1

rcall	Delay495
ret


;--------------------------------------